#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "Fl_Loop.H"

void Fl_Loop::draw()
{
  int dx, dy, dw, dh, sw, bx1, bx2, by, bh;
  double max;

  /* box borders */
  dx = Fl::box_dx(box());
  dy = Fl::box_dy(box());
  dw = Fl::box_dw(box());
  dh = Fl::box_dh(box());

  /* main box */
  fl_push_clip(x(), y(), w(), h());
  draw_box(box(), x(), y(), w(), h(), active_r() ? color() : fl_inactive(color()));
  fl_pop_clip();

  /* slider */
  if (value_ > 0.0 && value_ < 1.0) {
    sw = (int)(slider_size_ * w());
    max = w() + sw;
    bx1 = x() + dx;
    bx2 = bx1 + (int)(value_ * max) - sw;
    by = y() + dy;
    bh = h() - dh;

    fl_push_clip(bx1, by, w() - dw, bh);
    draw_box(FL_FLAT_BOX, bx2, by, sw - dw, bh,
             active_r() ? selection_color() : fl_inactive(selection_color()));
    fl_pop_clip();
  }

  /* label */
  Fl_Color col = labelcolor();
  labelcolor(fl_contrast(labelcolor(), selection_color()));

  fl_push_clip(x(), y(), w(), h());
  draw_label(x() + dx, y() + dy, w() - dw, h() - dh);
  fl_pop_clip();

  labelcolor(col);
}

void Fl_Loop::slider_size(double v) {
  slider_size_ = (v > 0.0 && v < 1.0) ? v : 0.2;
}

Fl_Loop::Fl_Loop(int x, int y, int w, int h, const char *l)
  : Fl_Widget(x, y, w, h, l)
{
  align(FL_ALIGN_INSIDE);
  box(FL_DOWN_BOX);
  color(fl_darker(color()));
  selection_color(FL_BLUE);
  value(0.0);
  slider_size(0.2);
}

